@@ -1,9 +1,4 @@
 //! A collection of reusable algorithms without dependencies on any other module in the project.
 
 pub mod clustering;
-pub mod gsom;
-pub mod mdp;
-pub mod nsga2;
-
 pub mod geometry;
-pub mod statistics;
