@@ -9,8 +9,8 @@ use crate::models::common::*;
 use crate::models::common::{Dimensions, ValueDimension};
 use crate::models::problem::{Actor, Job};
 use crate::models::Problem;
-use crate::utils::{unwrap_from_result, Environment};
 use hashbrown::HashSet;
+use rosomaxa::prelude::*;
 use std::cmp::Ordering;
 use std::ops::Deref;
 use std::sync::Arc;
