@@ -35,6 +35,10 @@ impl ConstraintModule for TourSizeModule {
 
     fn accept_solution_state(&self, _: &mut SolutionContext) {}
 
+    fn merge(&self, source: Job, _candidate: Job) -> Result<Job, i32> {
+        Ok(source)
+    }
+
     fn state_keys(&self) -> Iter<i32> {
         self.state_keys.iter()
     }
@@ -52,7 +56,7 @@ struct TourSizeHardRouteConstraint {
 impl HardRouteConstraint for TourSizeHardRouteConstraint {
     fn evaluate_job(&self, _: &SolutionContext, ctx: &RouteContext, job: &Job) -> Option<RouteConstraintViolation> {
         if let Some(limit) = self.limit_func.deref()(ctx.route.actor.as_ref()) {
-            let tour_activities = ctx.route.tour.activity_count();
+            let tour_activities = ctx.route.tour.job_activity_count();
 
             let job_activities = match job {
                 Job::Single(_) => 1,
