@@ -92,31 +92,26 @@
 
 extern crate rand;
 
-use crate::algorithms::nsga2::Objective;
-use crate::construction::Quota;
+use crate::construction::heuristics::InsertionContext;
 use crate::models::common::Cost;
+use crate::models::problem::ProblemObjective;
 use crate::models::{Problem, Solution};
-use crate::solver::population::Population;
-use crate::utils::{Environment, Timer};
+use crate::solver::search::Recreate;
 use hashbrown::HashMap;
+use rosomaxa::evolution::*;
+use rosomaxa::get_default_population;
+use rosomaxa::prelude::*;
 use std::any::Any;
 use std::sync::Arc;
 
-pub mod hyper;
-pub mod mutation;
+pub use self::heuristic::*;
+use rosomaxa::population::Rosomaxa;
+
 pub mod objectives;
-pub mod population;
 pub mod processing;
-pub mod termination;
+pub mod search;
 
-mod builder;
-pub use self::builder::Builder;
-
-mod evolution;
-use self::evolution::{EvolutionConfig, EvolutionSimulator};
-
-mod telemetry;
-pub use self::telemetry::{Metrics, Telemetry, TelemetryMode};
+mod heuristic;
 
 /// A key to store solution order information.
 const SOLUTION_ORDER_KEY: i32 = 1;
@@ -133,19 +128,16 @@ pub struct RefinementContext {
     pub problem: Arc<Problem>,
 
     /// A population which tracks best discovered solutions.
-    pub population: Box<dyn Population + Sync + Send>,
+    pub population: TargetPopulation,
 
     /// A collection of data associated with refinement process.
     pub state: HashMap<String, Box<dyn Any + Sync + Send>>,
 
-    /// A quota for refinement process.
-    pub quota: Option<Arc<dyn Quota + Send + Sync>>,
-
     /// An environmental context.
     pub environment: Arc<Environment>,
 
     /// A refinement statistics.
-    pub statistics: Statistics,
+    pub statistics: HeuristicStatistics,
 }
 
 /// Defines instant refinement speed type.
@@ -158,110 +150,140 @@ pub enum RefinementSpeed {
     Moderate,
 }
 
-/// A refinement statistics to track evolution progress.
-#[derive(Clone)]
-pub struct Statistics {
-    /// A number which specifies refinement generation.
-    pub generation: usize,
-
-    /// Elapsed seconds since algorithm start.
-    pub time: Timer,
-
-    /// A current refinement speed.
-    pub speed: RefinementSpeed,
-
-    /// An improvement ratio from beginning.
-    pub improvement_all_ratio: f64,
-
-    /// An improvement ratio for last 1000 iterations.
-    pub improvement_1000_ratio: f64,
-
-    /// A progress till algorithm's termination.
-    pub termination_estimate: f64,
-}
-
 impl RefinementContext {
     /// Creates a new instance of `RefinementContext`.
-    pub fn new(
-        problem: Arc<Problem>,
-        population: Box<dyn Population + Sync + Send>,
-        environment: Arc<Environment>,
-        quota: Option<Arc<dyn Quota + Send + Sync>>,
-    ) -> Self {
-        Self { problem, population, state: Default::default(), quota, environment, statistics: Statistics::default() }
+    pub fn new(problem: Arc<Problem>, population: TargetPopulation, environment: Arc<Environment>) -> Self {
+        Self { problem, population, state: Default::default(), environment, statistics: HeuristicStatistics::default() }
     }
 }
 
-impl Default for Statistics {
-    fn default() -> Self {
-        Self {
-            generation: 0,
-            time: Timer::start(),
-            speed: RefinementSpeed::Moderate,
-            improvement_all_ratio: 0.,
-            improvement_1000_ratio: 0.,
-            termination_estimate: 0.,
-        }
+impl HeuristicContext for RefinementContext {
+    type Objective = ProblemObjective;
+    type Solution = InsertionContext;
+
+    fn objective(&self) -> &Self::Objective {
+        self.problem.objective.as_ref()
+    }
+
+    fn population(&self) -> &(dyn HeuristicPopulation<Objective = Self::Objective, Individual = Self::Solution>) {
+        self.population.as_ref()
+    }
+
+    fn population_mut(
+        &mut self,
+    ) -> &mut (dyn HeuristicPopulation<Objective = Self::Objective, Individual = Self::Solution>) {
+        self.population.as_mut()
+    }
+
+    fn statistics(&self) -> &HeuristicStatistics {
+        &self.statistics
+    }
+
+    fn statistics_mut(&mut self) -> &mut HeuristicStatistics {
+        &mut self.statistics
+    }
+
+    fn environment(&self) -> &Environment {
+        self.environment.as_ref()
     }
 }
 
-/// A Vehicle Routing Problem Solver based on evolutionary algorithm.
+impl Stateful for RefinementContext {
+    type Key = String;
+
+    fn set_state<T: 'static + Send + Sync>(&mut self, key: Self::Key, state: T) {
+        self.state.insert(key, Box::new(state));
+    }
+
+    fn get_state<T: 'static + Send + Sync>(&self, key: &Self::Key) -> Option<&T> {
+        self.state.get(key).and_then(|v| v.downcast_ref::<T>())
+    }
+
+    fn state_mut<T: 'static + Send + Sync, F: Fn() -> T>(&mut self, key: Self::Key, inserter: F) -> &mut T {
+        // NOTE may panic if casting fails
+        self.state.entry(key).or_insert_with(|| Box::new(inserter())).downcast_mut::<T>().unwrap()
+    }
+}
+
+/// Wraps recreate method as `InitialOperator`
+pub struct RecreateInitialOperator {
+    recreate: Arc<dyn Recreate + Send + Sync>,
+}
+
+impl RecreateInitialOperator {
+    /// Creates a new instance of `RecreateInitialOperator`.
+    pub fn new(recreate: Arc<dyn Recreate + Send + Sync>) -> Self {
+        Self { recreate }
+    }
+}
+
+impl InitialOperator for RecreateInitialOperator {
+    type Context = RefinementContext;
+    type Objective = ProblemObjective;
+    type Solution = InsertionContext;
+
+    fn create(&self, heuristic_ctx: &Self::Context) -> Self::Solution {
+        let insertion_ctx = InsertionContext::new(heuristic_ctx.problem.clone(), heuristic_ctx.environment.clone());
+        self.recreate.run(heuristic_ctx, insertion_ctx)
+    }
+}
+
+/// Solves a Vehicle Routing Problem and returns a _(solution, its cost)_ pair in case of success
+/// or error description, if solution cannot be found.
+///
+/// A newly created builder instance is pre-configured with some reasonable defaults for mid-size
+/// problems (~200), so there is no need to call any of its methods.
+///
+///
+/// # Examples
+///
+/// This example shows how to construct default configuration for the solver, override some of default
+/// metaheuristic parameters using fluent interface methods, and run the solver:
+///
+/// ```
+/// # use vrp_core::models::examples::create_example_problem;
+/// # use std::sync::Arc;
+/// use vrp_core::prelude::*;
+///
+/// // create your VRP problem
+/// let problem: Arc<Problem> = create_example_problem();
+/// let environment = Arc::new(Environment::new_with_time_quota(Some(60)));
+/// // build solver config using pre-build builder with defaults and then override some parameters
+/// let config = create_default_config_builder(problem.clone(), environment)
+///     .with_max_time(Some(60))
+///     .with_max_generations(Some(100))
+///     .build()?;
+///
+/// // run solver and get the best known solution within its cost. Telemetry metrics are ignored.
+/// let (solution, cost, _) = Solver::new(problem, config).solve()?;
+///
+/// assert_eq!(cost, 42.);
+/// assert_eq!(solution.routes.len(), 1);
+/// assert_eq!(solution.unassigned.len(), 0);
+/// # Ok::<(), String>(())
+/// ```
 pub struct Solver {
-    /// A VRP problem definition.
-    pub problem: Arc<Problem>,
-    /// An evolution configuration.
-    pub config: EvolutionConfig,
+    problem: Arc<Problem>,
+    config: EvolutionConfig<RefinementContext, ProblemObjective, InsertionContext>,
 }
 
 impl Solver {
+    /// Tries to create an instance of `Solver` from provided config.
+    pub fn new(
+        problem: Arc<Problem>,
+        config: EvolutionConfig<RefinementContext, ProblemObjective, InsertionContext>,
+    ) -> Self {
+        Self { problem, config }
+    }
+
     /// Solves a Vehicle Routing Problem and returns a _(solution, its cost)_ pair in case of success
     /// or error description, if solution cannot be found.
-    ///
-    /// # Examples
-    ///
-    /// The most simple way to run solver is to use [`Builder`](./struct.Builder.html)
-    /// which has preconfigured settings:
-    ///
-    /// ```
-    /// # use vrp_core::models::examples::create_example_problem;
-    /// # use std::sync::Arc;
-    /// use vrp_core::solver::Builder;
-    /// use vrp_core::models::Problem;
-    /// use vrp_core::utils::Environment;
-    ///
-    /// // create your VRP problem
-    /// let problem: Arc<Problem> = create_example_problem();
-    /// let environment = Arc::new(Environment::default());
-    /// // build solver using builder with default settings
-    /// let solver = Builder::new(problem, environment).build()?;
-    /// // run solver and get the best known solution within its cost.
-    /// let (solution, cost, _) = solver.solve()?;
-    ///
-    /// assert_eq!(cost, 42.);
-    /// assert_eq!(solution.routes.len(), 1);
-    /// assert_eq!(solution.unassigned.len(), 0);
-    /// # Ok::<(), String>(())
-    /// ```
-    pub fn solve(self) -> Result<(Solution, Cost, Option<Metrics>), String> {
-        let mut config = self.config;
-        let processing = config.processing.clone();
-
-        config.problem = if let Some(processing) = &processing {
-            processing.pre_process(config.problem.clone(), config.environment.clone())
-        } else {
-            config.problem.clone()
-        };
-
-        let (population, metrics) = EvolutionSimulator::new(config)?.run()?;
+    pub fn solve(self) -> Result<(Solution, Cost, Option<TelemetryMetrics>), String> {
+        let (mut solutions, metrics) = EvolutionSimulator::new(self.config)?.run()?;
 
         // NOTE select the first best individual from population
-        let (insertion_ctx, _) = population.ranked().next().ok_or_else(|| "cannot find any solution".to_string())?;
-
-        let insertion_ctx = if let Some(processing) = processing {
-            processing.post_process(insertion_ctx.deep_copy())
-        } else {
-            insertion_ctx.deep_copy()
-        };
+        let insertion_ctx = if solutions.is_empty() { None } else { solutions.drain(0..1).next() }
+            .ok_or_else(|| "cannot find any solution".to_string())?;
 
         let solution = insertion_ctx.solution.to_solution(self.problem.extras.clone());
         let cost = self.problem.objective.fitness(&insertion_ctx);
