@@ -14,10 +14,12 @@ impl HeuristicSolutionProcessing for AdvanceDeparture {
         let mut insertion_ctx = solution.deep_copy();
 
         let problem = insertion_ctx.problem.clone();
-        let transport = problem.transport.clone();
+
+        let activity = problem.activity.as_ref();
+        let transport = problem.transport.as_ref();
 
         insertion_ctx.solution.routes.iter_mut().for_each(|route_ctx| {
-            TransportConstraintModule::advance_departure_time(route_ctx, transport.as_ref(), true);
+            TransportConstraintModule::advance_departure_time(route_ctx, activity, transport, true);
         });
 
         problem.constraint.accept_solution_state(&mut insertion_ctx.solution);
